User management microservice
============================

A simple user management microservice providing REST API based on JWT
token hand-offs.

Provides users with passwords, verified email addresses and roles, and
maintains a set of JWT tokens.

For talking to other microservices, uses a special-cased "@interconnect"
role which allows verification of JWT tokens provided by them and gives
unrestricted rights to manage all the tables.

Database model
--------------

User:
    * id INT AUTOINCREMENT
    * email VARCHAR NULL
    * name VARCHAR
    * password VARCHAR

EmailVerification:
    * user_id: FOREIGN KEY User(id)
    * email_address: VARCHAR
    * token: VARCHAR
    * expiry_date: VARCHAR

Role:
    * name VARCHAR

UserRole:
    * user_id: FOREIGN KEY User(id)
    * role_name: FOREIGN KEY Role(name)

HTTP API
--------

    GET /user — List users
    POST /user — Create user
    PATCH /user/<id> — Modify user

    GET /user/<id>/roles — List user's roles
    POST /user/<id>/roles — Add a role for a user
    DELETE /user/<id>/roles/<role-name> — Remove a role from a user

    GET /role — List available roles
    POST /role — Create a role
    GET /role/<name> — Get particular role
    PATCH /role/<name> — Update particular role

    GET /users/<id>/tokens


Deployment
----------
    * Set up Postgres and create a database and user:
        createuser -P $(DBUSER); createdb -O $(DBUSER) $(DBNAME)
    * Configure User management in users/src/calorify_users/config.py, esp. the database_url
    * Configure alembic.ini sqlalchemy.url to the same value as database_url
    * Configure virtualenv and install deps
        cd users
        virtualenv .venv
        source .venv/bin/activate
        pip install -r requirements.txt
        pip install -e .  # for scripts, otherwise not required
    * Set up database
        PYTHONPATH=src alembic upgrade head
    * Run the web app (preconfigured in the script on http://127.0.0.1:5000/)
        PYTHONPATH=src python src/calorify_users/app.py
    * Create basic users ("@admin" and "@manager" are hard-coded role names):
        python scripts/create-user.py --role '@admin' me@example.com admin Admin
        python scripts/create-user.py --role '@manager' someone@example.com manager Manager
        ...
    * Browse to http://127.0.0.1:5000/api/doc

Tests
-----

Ensure you've got test_database_url set to a user that has privileges to create
databases (eg. "createuser -s $USER" if you have not configured your local user
yet).

    $ pytest src

With coverage:

    $ pytest --cov=src --cov-report=term-missing

Multiple test runners (each gets their own independent database):

    $ pytest -n5 --cov=src --cov-report=term-missing
