import argparse
import logging
import sys

from sqlalchemy import create_engine
from sqlalchemy.orm import Session
from urllib.parse import urlencode

from calorify_users.config import config
from calorify_users.model import (
    Role,
    User,
    UserRole,
)


def get_token_for_user(email, password):
    import http.client
    import json
    import pprint

    connection = http.client.HTTPConnection('127.0.0.1:5000')
    connection.request(
        "POST", "/authenticate",
        body=urlencode({"email": email, "password": password}),
        headers={"Accept": "application/json",
                 "content-type": "application/x-www-form-urlencoded"})
    response = connection.getresponse()
    json_with_token = json.loads(response.read())
    token = json_with_token["token"]
    return token


def main():
    logger = logging.getLogger("calorify_users[create-user]")
    logger.setLevel(logging.INFO)
    logger.addHandler(logging.StreamHandler(sys.stderr))
    logger.warn("WARN: This script creates a user without any validation.")

    parser = argparse.ArgumentParser(description='Create a user')
    parser.add_argument('email', help='email address for user to be added')
    parser.add_argument('--role', action='append', help='roles to add for a user')
    parser.add_argument('password', help="user's password")
    parser.add_argument('name', help="user's name")

    args = parser.parse_args()
    if args.role is None:
        args.role = []
    print(args.role, args.email, args.password, args.name)

    engine = create_engine(config.get("database_url"))
    session = Session(bind=engine)

    existing_user = session.query(User).filter(
        User.email == args.email).one_or_none()
    if existing_user is None:
        logger.info("Creating user '{}' with roles [{}].".format(
            args.email, ", ".join(args.role)))
        # Creates a new user and their roles
        new_user = User(
            name=args.name, email=args.email, password=args.password)
        roles = [Role(name=role) for role in args.role]
        session.add_all([new_user, *roles])
        session.commit()
        session.add_all([
            UserRole(user_id=new_user.id, role_id=role.id)
            for role in roles])
        session.commit()
    else:
        user_exists_message = (
            "User '{}' already exists (with roles: [{}]), skipping creation.")
        logger.info(
            user_exists_message.format(
                existing_user.email,
                ", ".join([role.name for role in existing_user.roles])))
    logger.info("JWT token: '{}'.".format(
        get_token_for_user(args.email, args.password)))


if __name__ == "__main__":
    main()
