import crypt
import pytest

from calorify_users.model.emailverification import (
    EmailVerification,
)
from calorify_users.model.user import User


class TestUser:
    async def test_new_user(self, testdb_session):
        new_user = User(email="woo@hoo", name="Woo Hoo")
        testdb_session.add(new_user)
        testdb_session.commit()
        assert new_user.id is not None, "New user has not gotten an ID yet."

    async def test_verification_link(self, testdb_session):
        new_user = User(name="Woo Hoo")
        testdb_session.add(new_user)
        testdb_session.flush()
        verification = EmailVerification(
            user_id=new_user.id, email="woo@hoo", token="aa")
        testdb_session.add(verification)
        testdb_session.flush()

        assert new_user.email_verification == verification, (
            "Verification link not set correctly.")

    async def test_email_validation(self, testdb_session):
        at_sign_missing = "E-mail address must contain an '@' character."
        with pytest.raises(AssertionError) as error:
            User(email="woo.hoo", name="Woo Hoo")
        assert str(error.value) == at_sign_missing, (
            "Failed for a wrong reason.")

    async def test_password_hash(self, testdb_session):
        """Assert that password is correctly hashed when User is created."""
        password = "test"
        new_user = User(email="woo@hoo", name="Woo Hoo", password=password)
        testdb_session.add(new_user)
        testdb_session.commit()
        hash = new_user.password_hash
        assert hash == crypt.crypt(password, hash), (
            "Password has not been hashed correctly.")
