import pytest
import sqlalchemy.exc

from calorify_users.model.user import User
from calorify_users.model.userrole import UserRole
from calorify_users.model.role import Role


class TestUserRole:

    def test_new_role(self, testdb_session):
        """A new Role record can be added."""
        new_role = Role(name="admin")
        new_user = User(name="Woo Hoo", password="pwd")
        testdb_session.add_all([new_role, new_user])
        testdb_session.flush()

        user_role = UserRole(user_id=new_user.id, role_id=new_role.id)
        testdb_session.add(user_role)
        testdb_session.flush()
        assert user_role.added is not None, "Date and time not set."

        assert new_user.roles == [new_role]

    def test_already_exists(self, testdb_session):
        """When a user already has a role, re-adding it fails."""
        new_role = Role(name="admin")
        new_user = User(name="Woo Hoo", password="pwd")
        testdb_session.add_all([new_role, new_user])
        testdb_session.flush()

        user_role = UserRole(user_id=new_user.id, role_id=new_role.id)
        testdb_session.add(user_role)
        testdb_session.flush()

        with pytest.raises(sqlalchemy.exc.SQLAlchemyError):
            repeat_role = UserRole(user_id=new_user.id, role_id=new_role.id)
            testdb_session.add(repeat_role)
            testdb_session.flush()

    def test_same_role_multiple_users(self, testdb_session):
        """Ensure multiple users can get the same role."""
        new_role = Role(name="admin")
        new_user1 = User(name="Woo Hoo", password="pwd")
        new_user2 = User(name="Foo Bar", password="pwd")
        testdb_session.add_all([new_role, new_user1, new_user2])
        testdb_session.flush()

        user_role1 = UserRole(user_id=new_user1.id, role_id=new_role.id)
        user_role2 = UserRole(user_id=new_user2.id, role_id=new_role.id)
        testdb_session.add_all([user_role1, user_role2])
        testdb_session.flush()

        assert new_user1.roles == [new_role]
        assert new_user2.roles == [new_role]

    def test_one_user_multiple_roles(self, testdb_session):
        """Ensure multiple users can get the same role."""
        new_role1 = Role(name="admin")
        new_role2 = Role(name="user")
        new_user = User(name="Woo Hoo", password="pwd")
        testdb_session.add_all([new_role1, new_role2, new_user])
        testdb_session.flush()

        user_role1 = UserRole(user_id=new_user.id, role_id=new_role1.id)
        user_role2 = UserRole(user_id=new_user.id, role_id=new_role2.id)
        testdb_session.add_all([user_role1, user_role2])
        testdb_session.flush()

        assert new_user.roles == [new_role1, new_role2]
