import crypt

from sqlalchemy import (
    Column,
    Integer,
    String,
)
from sqlalchemy.orm import (
    relationship,
    validates,
)
from sqlalchemy.ext.hybrid import hybrid_property

from calorify_users.helpers import validate_email
from calorify_users.model import Base


class User(Base):
    __tablename__ = "user"

    id = Column(Integer(), primary_key=True)
    email = Column(String(), default=None, unique=True)
    name = Column(String(), default=None)
    password_hash = Column(String(128), default=None)
    email_verification = relationship(
        "EmailVerification", back_populates="user", uselist=False,
        cascade="delete")
    roles = relationship("Role", secondary="user_role", cascade="delete")

    @validates("email")
    def email_validator(self, key, value):
        return validate_email(value)

    @hybrid_property
    def password(self):
        return None

    @password.setter
    def password(self, new_value):
        self.password_hash = crypt.crypt(
            new_value, crypt.mksalt(crypt.METHOD_SHA512))
