from functools import wraps

from calorify_users.view.generic import error_response


class require_role:
    """
    Ensure `handler` is being accessed by an user with appropriate `role`.
    """

    def __init__(self, *roles):
        self.roles = set(roles)

    def __call__(self, handler):
        @wraps(handler)
        async def wrapper(request):
            authorized_roles = self.roles.intersection(
                set(request.get("authorized_roles", set())))
            if authorized_roles:
                return await handler(request)
            else:
                return error_response(
                    request, "Unauthorized", {}, status=401)
        return wrapper
