import pytest

from calorify_users.helpers import validate_email


class TestValidateEmail:

    def test_simple_email(self):
        email = validate_email("woo@hoo")
        assert email == "woo@hoo"

    def test_email_no_at_sign(self):
        at_sign_missing = "E-mail address must contain an '@' character."
        with pytest.raises(AssertionError) as error:
            validate_email("woo.hoo")
        assert str(error.value) == at_sign_missing, (
            "Failed for a wrong reason.")

    def test_no_at_sign(self):
        at_sign_missing = "E-mail address must contain an '@' character."
        with pytest.raises(AssertionError) as error:
            validate_email("woo.hoo")
        assert str(error.value) == at_sign_missing, (
            "Failed for a wrong reason.")

    def test_local_part_wrong(self):
        invalid_local_part = "Invalid e-mail address."
        with pytest.raises(AssertionError) as error:
            validate_email(".woo@hoo")
        assert str(error.value) == invalid_local_part, (
            "E-mail starting with dot should fail validation.")

    def test_domain_invalid(self):
        invalid_domain_part = "Invalid e-mail address."
        with pytest.raises(AssertionError) as error:
            validate_email("woo@hoo!")
        assert str(error.value) == invalid_domain_part, (
            "Domain part in email address should be a valid domain name.")

    def test_unicode_domain(self):
        email = validate_email("woo@шљњђ")
        assert email == "woo@шљњђ", "IDNA domains should work."

    def test_unicode_localpart(self, testdb_session):
        email = validate_email("шљњђ@hoo")
        assert email == "шљњђ@hoo", "Unicode is allowed for local part too."
