import pprint

from aiohttp import web


def error_response(request, error, hints, status=400):
    content_type = request.headers.get("ACCEPT", "application/json")
    error_data = {
        "message": error,
        "hints": hints,
    }
    if "json" in content_type:
        return web.json_response(error_data, status=status)
    else:
        return web.Response(text=error, status=status)


def adapt_response(request, data, status=200):
    """Adapt a response according to request Accept headers."""
    content_type = request.headers.get("ACCEPT", "application/json")
    if "json" in content_type:
        return web.json_response(data, status=status)
    else:
        return web.Response(
            text=pprint.pformat(data), status=status,
            headers={"content-type": "text/plain"})
