"""Fixtures and test classes for use in tests."""

import pytest
import base64
import os

from sqlalchemy import create_engine
from sqlalchemy.orm import Session
from sqlalchemy_utils.functions import (
    create_database,
    database_exists,
    drop_database,
)

from calorify_users.config import config
from calorify_users.model import Base


@pytest.fixture(scope="session")
def testdb():
    """Create a new unique test database and populate the schema."""
    db_url = config.get("test_database_url")
    while True:
        test_db_url = (
            db_url + "_test" + base64.b64encode(os.urandom(3)).decode("ascii"))
        if not database_exists(test_db_url):
            break
    # with open("/tmp/proba", "a") as output:
    #     output.write("CONSTRUCTING DB " + test_db_url + "...\n")
    create_database(test_db_url)
    yield test_db_url
    drop_database(test_db_url)


@pytest.fixture
def testdb_session(testdb):
    """Connects to the newly created database and clears the data."""
    engine = create_engine(testdb)
    Base.metadata.drop_all(engine)
    Base.metadata.create_all(engine)
    session = Session(bind=engine)
    yield session
    session.close()
